module Main where

import Simulation.Configuration.Parsing
import Simulation.Output(outputFromConfig)
import Simulation.Utility.ResultParser
import System.Console.GetOpt
import System.Environment(getArgs)
import System.Random(setStdGen, mkStdGen)
import qualified Simulation.Runnable as R

data CommandLineOpts = File FilePath | Display | Stats FilePath deriving (Show, Eq)

options :: [OptDescr CommandLineOpts]
options =
	[ Option ['f'] ["file"]    (ReqArg File "FILE")     "Configuration file",
	  Option ['g'] ["grapics"] (NoArg Display) "Run with graphics",
	  Option ['s'] ["statistics"] (ReqArg Stats "FILE") "Statistics to parse" ]

getPath [] = error "No file found in command-line options"
getPath (o:opts) = case o of
			(File p) -> p
			_	 -> getPath opts

getStats [] = (False, undefined)
getStats (x:xs) = case x of
			(Stats p) -> (True, p)
			_	  -> getStats xs

main = do 
	argv <- getArgs
	case getOpt Permute options argv of
		(o, _,[]  ) -> do
			let path  = getPath o
			let stats = getStats o
			config <- getJson path
			case config of
				(Right conf) ->	if elem Display o
						then R.runGraphically conf
						else if fst stats
						then outputRawData conf path $ snd stats
						else runSimulation' conf path
				(Left err)   -> error $
					"There was a problem parsing the settings file: " ++ err
		(_,_,errs) -> ioError (userError (concat errs ++ usageInfo "Usage:" options))

runSimulation' config path = do
	setStdGen $ mkStdGen $ seed config
	result <- R.runSimulation config
	let outputConfig = outputConf config
	outputFromConfig path outputConfig result

outputRawData config configPath dataPath = do
	result <- readRawStats dataPath
	let outputConfig = outputConf config
	outputFromConfig configPath outputConfig result
