{-# LANGUAGE BangPatterns #-}
module Simulation.Agents.LevyRandomAgent(generateLevyRandomAgent) where

import Graphics.Gloss.Data.Color
import Graphics.Gloss.Data.Picture
import Simulation.Common
import Simulation.Agents.SimpleAgent
import System.Random(RandomGen)
import Control.Monad.Random

data LevyRandomAgent = LRA {
	agent :: SimpleAgent
} deriving (Eq, Show)

convertToPower :: Float -> Float -> Float -> Float -> Float
convertToPower n x0 x1 r = (f1 - f0*r + f0) ** (1 / (n+1))
	where	f0 = x0 ** (n+1)
		f1 = x1 ** (n+1)

check :: (Num a, Ord a) => a -> a -> a
check b n = min b $ max (-b) n

checkBounds :: (Fractional a, Ord a, Num a) => a -> a -> a
checkBounds b = check (b/2)

selectNextPos :: (RandomGen g) => World -> SimpleAgent -> Rand g (Float, Float)
selectNextPos w a    = do 
	let (x, y)   = agentPos a
	let (wX, wY) = worldSize w
	r <- getRandomR (0, 2*pi)
	v <- getRandom
	let !l = convertToPower 2 5 150 v
	return $ (checkBounds wX (x + sin r * l), 
		checkBounds wY (y + cos r * l))

moveLevyAgent step w (LRA agent) = do 
	nAgent <- moveSimpleAgent selectNextPos step w agent
	return $ LRA nAgent

instance Agent LevyRandomAgent where
	update     = moveLevyAgent
	position a = [agentPos $ agent a]
	totalLen   = agentTravleLen . agent

instance Drawable LevyRandomAgent where
	draw = draw . agent

generateLevyRandomAgent :: (Float, Float) -> Float -> LevyRandomAgent
generateLevyRandomAgent pos sp = LRA $ createSimpleAgent pos pos sp
