{-# LANGUAGE BangPatterns, Rank2Types #-}
module Simulation.Agents.LevyRandomAgent2(
	generateLevyRandomAgent2, 
	levyRandom) where

import Control.Monad(replicateM, liftM)
import Control.Monad.Random(getRandomR, getRandom)
import GHC.Float(double2Float)
import Graphics.Gloss.Data.Color
import Graphics.Gloss.Data.Picture
import Simulation.Agents.SimpleAgent
import Simulation.Common

data LevyRandomAgent2 = LRA {
	agent 	  :: !SimpleAgent,
	levyAlpha :: !Double,
	levyScale :: !Double
} deriving (Eq, Show)

levyRandom :: Double -> SearchEnv Double
levyRandom my = do 
	u <- getRandom
	return $ (1 - u) ** (-1/my)
	
selectNextPos :: Double -> Double -> World -> SimpleAgent -> SearchEnv (Float, Float)
selectNextPos alpha scale w a    = do 
	let (x, y)   = agentPos a
	r <- getRandomR (0, 2*pi)
	l <- liftM (double2Float . (*scale)) $ levyRandom alpha
	let next = (x + cos r * l, y + sin r * l)
	return next

moveLevyAgent step w _ (LRA agent alpha scale) = do 
	nAgent <- moveSimpleAgent (selectNextPos alpha scale) step w agent
	return $ LRA nAgent alpha scale

instance Agent LevyRandomAgent2 where
	position a = [agentPos $ agent a]
	totalLen   = agentTravleLen . agent
	updateAgent= moveLevyAgent

instance Drawable LevyRandomAgent2 where
	draw = draw . agent

generateLevyRandomAgent2 :: Double -> Double -> (Float, Float) -> Float -> LevyRandomAgent2
generateLevyRandomAgent2 alpha scale pos sp = LRA (createSimpleAgent pos pos sp) alpha scale
