{-# LANGUAGE Rank2Types #-}
module Simulation.Agents.RandomAgent(RandomAgent, generateRandomAgent) where

import Control.Monad.Random(getRandomR)
import Graphics.Gloss.Data.Color
import Graphics.Gloss.Data.Picture
import Simulation.Agents.SimpleAgent
import Simulation.Common

data RandomAgent = RA {
	agent  :: !SimpleAgent
	--stride :: !Float	--How long should the movements become
} deriving (Eq, Show)

selectNextPos :: World -> SimpleAgent -> SearchEnv (Float, Float)
selectNextPos w a    = do 
	let (x, y)   = agentPos a
	let (wx, wy) = worldSize w
	r <- getRandomR (0, 2*pi)
	let v = sqrt $ wx**2 + wy ** 2
	let next = (x + cos r * v, y + sin r * v)
	return next

moveRandomAgent step w _ (RA agent) = do
	nAgent <- moveSimpleAgent selectNextPos  step w agent
	return $ RA nAgent

instance Drawable RandomAgent where
	draw = draw . agent

instance Agent RandomAgent where
	position a = [agentPos $ agent a]
	totalLen   = agentTravleLen . agent
	updateAgent= moveRandomAgent

generateRandomAgent :: (Float, Float) -> Float -> RandomAgent
generateRandomAgent pos sp = RA (createSimpleAgent pos pos sp)
