module Simulation.Agents.SimpleAgent(
	SimpleAgent(..), 
	moveSimpleAgent, 
	createSimpleAgent,
	createSimpleAgent') where

import Control.Monad(liftM)
import Graphics.Gloss.Data.Color
import Graphics.Gloss.Data.Picture
import Simulation.Common

data SimpleAgent = SA {
	agentDest	:: !(Float, Float),
	agentPos	:: !(Float, Float),
	agentSpeed	:: !Float,
	agentTravleLen  :: !Float
} deriving (Eq, Show)

createSimpleAgent dest pos speed = SA dest pos speed 0
createSimpleAgent' pos speed = SA pos pos speed 0

drawSimpleAgent :: SimpleAgent -> Picture
drawSimpleAgent (SA (x2, y2) (x, y) _ _)  = Pictures [drawAgent, drawLine]
	where	drawAgent = Color red $ Translate x y $ Circle agentSize
		drawLine  = Color green $ Line [(x, y), (x2, y2)]

instance Drawable SimpleAgent where
	draw = drawSimpleAgent

moveSimpleAgent :: 	(Monad m) 					=>
			(World -> SimpleAgent -> m (Float, Float)) 	-> 
			Float 						-> 
			World						->
			SimpleAgent 					-> 
			m SimpleAgent
moveSimpleAgent selectNextPos step world a@(SA (x2, y2) (x, y) sp len) =
	if abs (x2 - x) <= agentCloseThreshold && abs (y2 - y) <= agentCloseThreshold
	then do next <- selectNextPos world a
		if next /= (x2, y2)
		then moveSimpleAgent selectNextPos step world (SA next (x, y) sp len)
		else return a
	else 	if not $ insideWorld world (nX, nY)
		then return $ SA (x, y) (x, y) sp newLen
		else return $ SA (x2, y2) (nX, nY) sp newLen
	where	nX  = x + sp * l * nnX * step
		nnX = x2 - x
		nY  = y + sp * l * nnY * step
		nnY = y2 - y
		l   = 1 / sqrt (nnX ** 2 + nnY ** 2)
		newLen = len + sqrt ((nX - x) ** 2 + (nY - y) ** 2)
