module Simulation.Configuration.Configuration(RunnableModel(..), 
	ConfigSearchModel(..),
	parseConfiguration) where

import Simulation.Common(World, Agent, Transmitter)
import Simulation.Configuration.Parsing(getJson, ParsedConfig(..), 
	ParsingAgent, ParsingTransmitter)
import System.Random(StdGen, mkStdGen, setStdGen, newStdGen)
import Control.Monad(replicateM)

data RunnableModel = RM {
	name			:: String,
	agentModel		:: ParsingAgent,
	numberAgents		:: Int,
	transmitterModel	:: ParsingTransmitter
} 

data ConfigSearchModel = SM {
	initialSeed		:: Int,
	worldModel		:: World,
	startPosition		:: (Float, Float),
	smAgentSpeed		:: Float,
	numSimulations		:: Int,
	model			:: RunnableModel
}

parseConfiguration :: FilePath -> IO [ConfigSearchModel]
parseConfiguration configFile = do
	conf <- getJson configFile
	case conf of
		Nothing -> error "Configuration parsing failed"
		Just c  -> do
			let sims  = simulations c
			let w	  = world c
			let start = (-300, -300)
			let speed = agentSpeed c
			let nAgs  = numAgents c
			let trans = transmitter c
			let ags   = agents c
			setStdGen $ mkStdGen $ seed c
			gens <- replicateM (length nAgs * length ags) newStdGen
			undefined
