module Simulation.Graphics.Chart(renderResults) where

import Control.Lens
import Data.Colour
import Data.Default.Class
import Graphics.Rendering.Chart
import Graphics.Rendering.Chart.Gtk

chart line name color = [Left (toPlot line1), Left (toPlot err)]
	where	err   = plot_errbars_values .~ [symErrPoint x y 0 z | (x, (y, z)) <- line]
			$ plot_errbars_title .~ "Standard deviation for " ++ name
			$ def
		line1 = plot_lines_style . line_color .~ color
			$ plot_lines_values .~ [[ (x,y) | (x, (y, z)) <- line]]
			$ plot_lines_title .~ "Average for " ++ name
			$ def

chart2 lines names = [Left (plotBars line1)]
	where line1 = plot_bars_titles .~ names
		$ plot_bars_values  .~ addIndexes lines
		$ plot_bars_style   .~ BarsClustered
		$ plot_bars_spacing .~ BarsFixGap 30 5
		$ plot_bars_item_styles .~ map mkStyle (cycle defaultColorSeq)
		$ def

mkStyle c = (solidFillStyle c, Nothing)

createLayout plots = toRenderable layout
	where	layout = layout1_title .~ "Simulation results"
			$ layout1_plots .~ plots
			$ def

renderResults :: [(String, [(Int, (Double, Double))])] -> IO ()
renderResults d    = renderableToWindow l 800 800
	where 	l  = createLayout $ concat 
			$ map (\((name, line), c) -> chart line name c)
			$ zip d (cycle defaultColorSeq)
		l2 = createLayout $ chart2 barsAvg barsNam
		labels = map show $ extractIndex $ head d
		barsAvg= map extractAvg d
		barsNam= map extractName d

extractName (s, _) = s

extractAvg :: (String, [(Int, (Double, Double))]) -> [Double]
extractAvg (_, dat) = map (fst . snd) dat

extractIndex (_, dat) = map fst dat
