{-# LANGUAGE FlexibleContexts #-}
module Simulation.Graphics.Gnuplot(histogram2d, plotToScreen, plotToEps) where

import Data.Foldable (foldMap, )
import Graphics.Gnuplot.Plot.TwoDimensional (linearScale, )
import Graphics.Gnuplot.Terminal.PostScript(cons, color, eps)
import System.Exit(ExitCode)
import qualified Graphics.Gnuplot.Advanced as GP
import qualified Graphics.Gnuplot.Display as Display
import qualified Graphics.Gnuplot.Frame as Frame
import qualified Graphics.Gnuplot.Frame.OptionSet as Opts
import qualified Graphics.Gnuplot.Frame.OptionSet.Histogram as Histogram
import qualified Graphics.Gnuplot.Frame.OptionSet.Style as OptsStyle
import qualified Graphics.Gnuplot.Graph.TwoDimensional as Graph2D
import qualified Graphics.Gnuplot.LineSpecification as LineSpec
import qualified Graphics.Gnuplot.Plot.TwoDimensional as Plot2D

histogram2d :: String -> [(String, Int)] -> [(String, [Double])] -> Frame.T (Graph2D.T Int Double)
histogram2d title xRange values =
   Frame.cons (
      Opts.title title $
      Histogram.clusteredGap 2 $
      Opts.boxwidthAbsolute 0.9 $
      OptsStyle.fillBorderLineType (-1) $
      OptsStyle.fillSolid $
      Opts.xTicks2d xRange $
      Opts.deflt) $
   foldMap (\(title,dat) ->
      fmap (Graph2D.lineSpec (LineSpec.title title LineSpec.deflt)) $
      Plot2D.list Graph2D.histograms dat) values

plotToScreen :: (Display.C (Frame.T a)) => Frame.T a -> IO ExitCode
plotToScreen = GP.plotDefault

plotToEps :: (Display.C (Frame.T a)) => FilePath -> Frame.T a -> IO ExitCode
plotToEps path = GP.plot (eps $ color $ cons $ path ++ ".eps")
