{-# LANGUAGE BangPatterns, Rank2Types #-}
module Simulation.Model where

import Control.Monad(liftM)
import Control.Monad.Random(evalRandIO)
import Graphics.Gloss.Data.Color(black, white)
import Graphics.Gloss.Data.Display(Display(InWindow))
import Graphics.Gloss.Data.Picture
import Graphics.Gloss.Interface.IO.Simulate(simulateIO)
import Simulation.Common
import Simulation.Simulator(simulateUntil)
import Control.DeepSeq

data SearchModel a b = Search {
	numSteps 	:: !Int,
	searchTrans	:: !b,
	searchAgents	:: ![a],
	searchWorld 	:: !World
}

data SearchModelStatistics = SMS {
	totalSteps	:: !Int,
	totalLength	:: !Float
} deriving (Eq, Show, Read)

instance NFData SearchModelStatistics where

drawSearchModelM :: (Agent a, Transmitter b, Monad m) => SearchModel a b -> m Picture
drawSearchModelM model = return $ Pictures $ tDrawing : wDrawing : aDrawings
	where	tDrawing  = draw $ searchTrans model
		wDrawing  = Color black $ rectangleWire x y
		aDrawings = map draw $ searchAgents model
		(x, y)    = worldSize $ searchWorld model

updateSearchModel :: 	(Agent a, Transmitter b)=>
			Float -> SearchModel a b -> SearchEnv (SearchModel a b)
updateSearchModel step (Search n t ags w) = do
	!updateT   <- updateTrans step w t
	!updateAgs <- mapM (updateAgent step w t) ags
	return $ Search (n+1) updateT updateAgs w

updateSearchModelIO step !model = evalRandIO (updateSearchModel step model)

checkSearchModel :: (Agent a, Transmitter b) => SearchModel a b -> SearchEnv Bool
checkSearchModel !model = liftM or $ mapM (withinRange t) ags
	where	 !ags = searchAgents model
		 !t   = searchTrans  model

createSearchModel :: (Agent a, Transmitter b) => World -> b -> [a] -> SearchModel a b
createSearchModel w t ags = Search 0 t ags w

getStatistics :: (Agent a, Transmitter b) => SearchModel a b -> SearchModelStatistics
getStatistics model = SMS nS tL
	where	!nS = numSteps model
		!tL = sum $ map totalLen $ searchAgents model

runSearchModel :: (Agent a, Transmitter b) => SearchModel a b -> SearchEnv SearchModelStatistics
runSearchModel !m = do 
	!final <- simulateUntil 60 m checkSearchModel updateSearchModel
	return $! getStatistics final

runSearchModelGraphically :: (Agent a, Transmitter b) => SearchModel a b -> IO ()
runSearchModelGraphically m = do
	let display = InWindow "Simulation" (800, 800) (0, 0)
	simulateIO display white 60 m drawSearchModelM $ const updateSearchModelIO
