{-# LANGUAGE Rank2Types #-}
module Simulation.Transmitters.AlphaTransmitter(
	AlphaTransmitter(transPos),
	generateAlphaTransmitter) 
where

import Control.Monad(liftM)
import Data.Complex(Complex(..), magnitude)
import GHC.Float(float2Double, double2Float)
import Graphics.Gloss.Data.Color
import Graphics.Gloss.Data.Picture
import Simulation.Common

data AlphaTransmitter = Alpha{
	transPos  :: !(Float, Float),		--Position
	radius	  :: !Float,			--Range of transmitter
	alpha	  :: !Double,			--Alpha value
	amplitude :: !Double,			--Amplitude for transmitter
	sigma	  :: !Double,			--Sigma used to generate noise
	threshold :: !Double			--Threshold for false alarm rate
} deriving (Eq, Show)

generateAlphaTransmitter pos effect alpha sigma deg = Alpha pos radius alpha effect sigma threshold
	where	threshold = calculateTreshold sigma deg
		radius    = double2Float $ (effect / threshold) ** (2/alpha)

complexFromGaussian mu sigma = do
	i <- gaussian mu sigma
	q <- gaussian mu sigma
	return $ i :+ q

transmitterColor = mixColors 0.5 0.5 orange white

receivedSignal' t dist = amplitude t / sqrt (dist ** alpha t)

--Constant µ=0 for noise
generateGausNoise t = gaussian 0 (sigma t)
generateAmplitudeNoise t = liftM magnitude $ complexFromGaussian 0 (sigma t)

calculateTreshold :: Double -> Double -> Double
calculateTreshold sigma deg = sqrt $ (-2) * (sigma ** 2) * log (1 - deg)

instance Drawable AlphaTransmitter where
	draw t = Translate x y $ Pictures [color transmitterColor $ circleSolid $ radius t, color black $ circleSolid 5]
		where (x, y) = transPos t

instance Transmitter AlphaTransmitter where
	transmitterPos = transPos
	receivedSignal t dist = do 
		let received = receivedSignal' t dist
		noise' <- generateAmplitudeNoise t
		return $ received + noise'
	withinRange t ag = liftM or $ mapM (\(signal, dist) -> do
				noise <- generateAmplitudeNoise t
				return $ signal + noise > threshold t && 
					dist < radius t) signDist
		where   signDist = zip signs dists
			signs    = map (receivedSignal' t . float2Double) dists
			dists    = map (distance (transPos t)) $ position ag
	updateTrans _ _ = return
