{-# LANGUAGE BangPatterns #-}
module Simulation.Transmitters.SimpleTransmitter where

import Control.Monad.Random
import Graphics.Gloss.Data.Color
import Graphics.Gloss.Data.Picture
import Simulation.Common

--A fixed radius simple transmitter
data SimpleTransmitter = ST {
	radius 	 :: !Float,
	transPos :: !(Float, Float)
} deriving (Eq, Show)

instance Drawable SimpleTransmitter where
	draw (ST r (x, y)) = Color orange $ Translate x y $ Circle r

instance Updateable SimpleTransmitter where
	update _ _ = return . id

instance Transmitter SimpleTransmitter where
	receivedSignal _ _ = return 0
	withinRange t ag = return $ any (< agentSize + rad) ds
		where	ds 	  = map (distance (transPos t)) positions
			positions = position ag
			rad	  = radius t

generateSimpleTransmitter r pos = ST r pos

generateSimpleTransmitterM r (xSize, ySize) = do
	x <- getRandomR (-xSize / 2 + r, xSize / 2 - r)
	y <- getRandomR (-ySize / 2 + r, ySize / 2 - r)
	return $ generateSimpleTransmitter r (x, y)
