module Simulation.Utility.GridMap(
	GridMap(size),
	convertWith,
	createGridMap,
	getValue,
	updateGrid
where

import qualified Data.Vector.Unboxed as U

--A quadratic representation of a map containing values
data GridMap a = GridMap {
	grid :: !U.Vector a,
	size :: !Int
}

createGridMap :: U.Unbox a 	=> 
		 Int 		-> -- ^Grid size, will become size/res ** 2
		 Float 		-> -- ^Grid resolution, used to determine size
		 a 		-> -- ^Initial value to be replicated in map
		 GridMap a
createGridMap size res init = GridMap (U.replicate (s**2) init) s
	where s = round $ fromIntegral size / res

index :: GridMap a -> (Int, Int) -> Int
index (GridMap _ size) (x, y) = x + y * size
{-# INLINE index #-}

--Updates the grid with the new value adding it to the old value
updateGrid :: (U.Unbox a) 	=>
	      GridMap a		->
	      [((Int, Int), a)] ->
	      GridMap a
updateGrid m@(GridMap vec _) updates = U.accum (+) vec converted
	where converted = map (\(pos, val) -> (index m pos, val)) updates
{-# INLINE updateGrid #-}

getValue :: U.Unbox a 	=>
	    GridMap a	->
	    (Int, Int)	->
	    a		->
getValue m@(GridMap vec _) pos = 
	let i = index m pos
	in  vec U.! i
{-# INLINE getValue #-}

convertWith ::	(U.Unbox a, U.Unbox b)	=>
	  	(a -> b)		->
	  	GridMap a		->
	  	GridMap b
convertWith f (GridMap vec size) = 
	let newVec = U.map f vec
	in  GridMap newVec size
