module Simulation.Utility.Sampling(
	SamplingAvg,
	addSample,
	avg,
	clearSamples,
	createSamplingAvg,
	desiredCount,
	getSamples,
	isReady
) where

import Control.DeepSeq(NFData, force)

data SamplingAvg a = SAvg {
	numSamples	:: !Int, 	--Number of samples needed
	addedSamples	:: !Int,	--Used to count how many are newly added
	samples		:: ![a]		--Samples
} deriving (Eq, Show)

createSamplingAvg numS = SAvg numS 0 []

--Is there enough samples to calculate avg?
isReady :: SamplingAvg a -> Bool
isReady (SAvg num added _ ) = added >= num

--Add a new sample to the collection
addSample :: (NFData a) => SamplingAvg a -> a -> SamplingAvg a
addSample (SAvg n add samps) !new = SAvg n (add + 1) $ force (new : take (2 * n) samps)

--Calculate the average
avg :: (Fractional a) => SamplingAvg a -> a
avg sam = sum samplings / len
	where	samplings = take (numSamples sam) $ samples sam
		len	  = fromIntegral $ length samplings

--Clear the added sample counter
clearSamples :: SamplingAvg a -> SamplingAvg a
clearSamples sam = sam{addedSamples = 0}

desiredCount :: SamplingAvg a -> Int
desiredCount = numSamples

getSamples :: SamplingAvg a -> [a]
getSamples sam = take (desiredCount sam) $ samples sam
