module Simulation.Utility.Vec2 where

import Data.Foldable(foldr')
import Control.DeepSeq(force, NFData)

type Vec2 a = (a, a)

vec2Zero :: (Num a, NFData a) => Vec2 a
vec2Zero = (0, 0)

{-# INLINE vec2Add #-}
vec2Add :: (Num a, NFData a) => Vec2 a -> Vec2 a -> Vec2 a
vec2Add (!a1, !b1) (!a2, !b2) = force (a1 + a2, b1 + b2)

{-# INLINE vec2Sub #-}
vec2Sub :: (Num a, NFData a) => Vec2 a -> Vec2 a -> Vec2 a
vec2Sub (!a1, !b1) (!a2, !b2) = force (a1 - a2, b1 - b2)

{-# INLINE vec2Scale #-}
vec2Scale :: (Num a, NFData a) => a -> Vec2 a -> Vec2 a
vec2Scale num (!a, !b) = force (a*num, b*num)

{-# INLINE vec2Sum #-}
vec2Sum :: (Num a, NFData a) => [Vec2 a] -> Vec2 a
vec2Sum = force . foldr' vec2Add vec2Zero

{-# INLINE vec2Len #-}
vec2Len :: (Floating a, NFData a) => Vec2 a -> a
vec2Len (!a, !b) = force $ sqrt (a**2 + b**2)

{-# INLINE vec2Normalize #-}
vec2Normalize :: (Floating a, Ord a, NFData a) => Vec2 a -> Vec2 a
vec2Normalize !v = if len > 0 then force $ vec2Scale (1/len) v else vec2Zero
	where	len = vec2Len v
